/**
 * File for utility logic.
 */

// User Import Headers.
#ifndef user_headers
    #define user_headers
    #include "utility.h"
#endif

#ifndef user_headers_logging
    #define user_headers_logging
    #include "logging.h"
#endif


// Global Variables.
#ifndef LOGGING_CLASS_INIT
    #define LOGGING_CLASS_INIT
    extern Logging logger;
#endif
const int TYPE_DIR = 0;
const int TYPE_FILE = 1;
const int TYPE_OTHER = 2;
path_validator_struct* path_validator_ptr;


/**
 * Initializes a "path_validator" object.
 */
path_validator_struct* initialize_path_validator_struct(int file_type, std::error_code err_code) {
    logger.debug("initialize_path_validator_struct():");
    logger.debug("    file_type: " + std::to_string(file_type));
    // logger.debug("    err_code: " + err_code);
    logger.debug("");

    // Create struct.
    path_validator_ptr = (path_validator_struct*) calloc(1, sizeof(path_validator_struct));

    // Populate fields.
    path_validator_ptr->file_type = file_type;
    path_validator_ptr->err_code = err_code;

    return path_validator_ptr;
}


/**
 * Destroyes passed "path_validator" object and all associated fields.
 */
void free_path_validator_struct(path_validator_struct* path_validator_ptr) {
    logger.debug("free_path_validator_struct():");
    // logger.debug("    path_validator_ptr: " + path_validator_ptr);
    logger.debug("");

    free(path_validator_ptr);
}


/**
 * Validates provided path and if valid, determines path type.
 * Returns struct of data.
 */
path_validator_struct* validate_path(std::string path_str) {
    logger.debug("validate_path():");
    logger.debug("    path_str: " + path_str);
    logger.debug("");

    int type_value = TYPE_OTHER;
    std::error_code ec;

    // Check if provided path exists.
    std::ifstream test_path_exists(path_str);
    if (test_path_exists) {
        // Path exists. Check if dir or file.
        if (std::filesystem::is_directory(path_str, ec)) {
            // Is dir.
            type_value = TYPE_DIR;
        } else if ((! ec) && (std::filesystem::is_regular_file(path_str, ec))) {
            // Is file.
            type_value = TYPE_FILE;
        }
    }

    // Return values.
    path_validator_struct* return_struct_ptr = initialize_path_validator_struct(type_value, ec);
    return return_struct_ptr;
}
